import base64
from io import BytesIO

from psd_tools.api.layers import Layer

from psd2fabric.fabric.image import ImageFabricLayer


def parse(layer: Layer, relate_x, relate_y):
    image = layer.composite()

    # image -> base64
    image_stream = BytesIO()
    image.save(image_stream, format="png")
    src = "data:image/png;base64," + base64.b64encode(image_stream.getvalue()).decode('utf-8')

    return ImageFabricLayer(layer.name, layer.left - relate_x, layer.top - relate_y, layer.width, layer.height, src)



from psd_tools.api.effects import ColorOverlay

from psd2fabric.fabric import FabricLayer


def coloroverlay_parse(coloroverlay: ColorOverlay, fabric_layer: FabricLayer):
    fabric_layer.opacity = round(coloroverlay.opacity / 100, 2)
    # if not fabric_layer.filters:
    #     fabric_layer.filters = []
    #
    # fabric_layer.append("{}")


import uuid


class FabricLayer:
    def __init__(self, name, type, left, top, width, height):
        self.type = type
        self.version = "5.3.0"
        self.originX = "left"
        self.originY = "top"
        self.left = left
        self.top = top
        self.width = width
        self.height = height
        self.scaleX = 1
        self.scaleY = 1
        self.angle = 0
        self.flipX = False
        self.flipY = False
        self.opacity = 1
        self.visible = True
        self.backgroundColor = ""
        self.skewX = 0
        self.skewY = 0
        self.selectable = True
        self.hasControls = True
        self.id = str(uuid.uuid4())
        self.name = name

    def set_stroke(self, color, size):
        self.stroke = color
        self.strokeWidth = size


class Fabric:
    def __init__(self, objs, left, top, width, height):
        self.version = "5.3.0"
        self.objects = []
        self.clipPath = {
            "type": "rect",
            "version": "5.3.0",
            "originX": "left",
            "originY": "top",
            "left": left,
            "top": top,
            "width": width,
            "height": height
        }

        workspace = {
            "type": "rect",
            "version": "5.3.0",
            "originX": "left",
            "originY": "top",
            "left": left,
            "top": top,
            "width": width,
            "height": height,
            "fill": "#FC7245",
            "id": "workspace",
            "selectable": False,
            "hasControls": False
        }

        self.objects.append(workspace)
        for obj in objs:
            # print(obj)
            self.objects.append(obj)



from psd2fabric.fabric import FabricLayer


class ImageFabricLayer(FabricLayer):
    def __init__(self, name, left, top, width, height, img):
        # 调用父类的构造方法
        super().__init__(name, "image", left, top, width, height)
        self.src = None
        self.set_image(img)

    def set_image(self, src: str):
        # 将图像转换为Base64编码
        self.src = src



from psd_tools.api.layers import TypeLayer

from psd2fabric.fabric.text import TextFabricLayer


def parse(layer: TypeLayer, relate_x, relate_y):
    text = layer.engine_dict['Editor']['Text'].value
    fontset = layer.resource_dict['FontSet']
    styleSheetSet = layer.resource_dict['StyleSheetSet']
    engineDict = layer.engine_dict
    runlength = engineDict["StyleRun"]["RunLengthArray"]
    rundata = engineDict["StyleRun"]["RunArray"]
    paragraph_rundata = engineDict['ParagraphRun']['RunArray']
    writingDirection = engineDict["Rendered"]["Shapes"]["WritingDirection"]
    index = 0
    for length, style, paragraph in zip(runlength, rundata, paragraph_rundata):
        # just use the first one
        # substring = text[index:index + length]
        stylesheet = style['StyleSheet']['StyleSheetData']
        paragraphsheet = paragraph['ParagraphSheet']['Properties']
        if 'Font' in stylesheet:
            fontType = stylesheet['Font']
        else:
            fontType = styleSheetSet[index]['StyleSheetData']['Font']

        font_size = stylesheet['FontSize']
        font_size = round(get_size(font_size, layer.transform), 2)
        font_name = fontset[fontType]['Name']
        font_color = get_color(stylesheet['FillColor']['Values'])
        break

    tlayer = TextFabricLayer(layer.name, layer.left - relate_x, layer.top - relate_y, layer.width, layer.height)
    text = get_text(text)
    tlayer.set_text(
        font_name,
        font_size,
        font_color,
        get_bold(stylesheet),
        get_align(paragraphsheet),
        # writingDirection = 2 表明是竖版字，使用换行符实现竖版字
        text if writingDirection != 2 else "\n".join(list(text.replace("\n", "")))
    )
    return tlayer


def get_color(color):
    rgba_values = [round(c * 255, 0) for c in color]
    return f"rgba({','.join(map(str, rgba_values[1:]))},{rgba_values[0]})"


def get_size(font_size, transform):
    return font_size * transform[0]

def get_text(text):
    return text.replace('\r', '\n')

def get_align(paragraphsheet):
    if not 'Justification' in paragraphsheet:
        return 'left'

    if paragraphsheet['Justification'] == 1:
        return 'right'
    elif paragraphsheet['Justification'] == 2:
        return 'center'

    return 'left'

def get_bold(stylesheet):
    if 'FauxBold' in stylesheet:
        return stylesheet['FauxBold']
    return False


import json

from psd2fabric.fabric import FabricLayer, Fabric


def custom_default(obj):
    if isinstance(obj, (FabricLayer, Fabric)):
        return obj.__dict__
    return str(obj)


def render_json(obj: Fabric):
    # 将 Person 对象转换为 JSON 字符串
    return json.dumps(obj, default=custom_default, ensure_ascii=False)


def dump_json_file(obj, file):
    content = render_json(obj)
    with open(file, 'w') as file:
        # 将字符串写入文件
        file.write(content)




from psd_tools import PSDImage

from psd2fabric.parser.psd_parser import psd_to_fabric
from psd2fabric.render.json_render import dump_json_file


def dump_psd(psd_file, dump_file):
    psd = PSDImage.open(psd_file)
    fabric = psd_to_fabric(psd)
    dump_json_file(fabric,dump_file)







from psd2fabric.fabric import FabricLayer


class TextFabricLayer(FabricLayer):
    def __init__(self, name, left, top, width, height):
        # 调用父类的构造方法
        super().__init__(name, "i-text", left, top, width, height)
        self.pathAlign = None
        self.pathSide = None
        self.pathStartOffset = None
        self.direction = None
        self.styles = None
        self.charSpacing = None
        self.textBackgroundColor = None
        self.lineHeight = None
        self.fontStyle = None
        self.textAlign = None
        self.linethrough = None
        self.overline = None
        self.underline = None
        self.fontWeight = None
        self.fontSize = None
        self.fontFamily = None

    def set_text(self, font_family, font_size, font_color, text_bold, text_align, text):
        self.fill = font_color
        self.fontFamily = font_family
        self.fontSize = font_size
        self.paintFirst = "stroke" # "fill"内边框
        self.text = text
        self.fontWeight = "normal"
        if text_bold:
            self.fontWeight = "bold"

        self.underline = False
        self.overline = False
        self.linethrough = False
        self.textAlign = text_align
        self.fontStyle = "normal"
        self.lineHeight = 0.86
        self.textBackgroundColor = ""
        self.charSpacing = 0
        self.styles = []
        self.direction = "ltr"
        self.pathStartOffset = 0
        self.pathSide = "left"
        self.pathAlign = "baseline"



from setuptools import setup, find_packages

setup(
    name='psd2fabric',
    version='0.1.1',
    packages=find_packages(),
    install_requires=[
        "psd_tools==1.9.28"
    ],
    entry_points={
        'console_scripts': [
            'psd2fabric = main'
        ]
    }
)



from psd2fabric.fabric import FabricLayer


class GroupFabricLayer(FabricLayer):
    def __init__(self, name, left, top, width, height):
        # 调用父类的构造方法
        super().__init__(name, "group", left, top, width, height)
        self.fill = "null"
        self.fillRule = "nonzero"
        self.paintFirst = "fill"
        self.globalCompositeOperation = "source-over"
        self.objects = []

    def add(self, objs: list):
        for obj in objs:
            # print(obj)
            self.objects.append(obj)


from psd_tools.api.effects import Stroke

from psd2fabric.fabric import FabricLayer


#
def stroke_parse(stroke: Stroke, fabric_layer: FabricLayer):
    r = int(stroke.color['Rd  '])
    g = int(stroke.color['Grn '])
    b = int(stroke.color['Bl  '])
    a = int(stroke.opacity)
    fabric_layer.set_stroke(f"rgba({r},{g},{b},{a})", stroke.size)



from psd_tools.api.effects import Effects, Stroke, ColorOverlay
from psd_tools.api.layers import AdjustmentLayer, FillLayer, Layer

from psd2fabric.fabric import Fabric, FabricLayer
from psd2fabric.fabric.group import GroupFabricLayer
from psd2fabric.parser import type_parser, image_parser
from psd2fabric.parser.effects.coloroverlay_parser import coloroverlay_parse
from psd2fabric.parser.effects.stroke_parser import stroke_parse


def parse_layers(psd_layers: list, relate_x, relate_y) -> list:
    res = []
    for layer in psd_layers:
        if not layer.visible:
            continue
        # 复杂图,暂时跳过
        elif isinstance(layer, AdjustmentLayer) or isinstance(layer, FillLayer):
            continue

        if layer.is_group():
            fabric_layer = GroupFabricLayer(layer.name, layer.left - relate_x, layer.top - relate_y, layer.width,
                                            layer.height)
            # group 内元素(left, top)默认是相对group center的
            children = parse_layers(layer._layers, layer.left + layer.width // 2, layer.top + layer.height // 2)
            fabric_layer.add(children)
            common_parse(layer, fabric_layer)
        elif layer.kind == 'type':
            fabric_layer = type_parser.parse(layer, relate_x, relate_y)
            common_parse(layer, fabric_layer)
        else:
            fabric_layer = image_parser.parse(layer, relate_x, relate_y)

        if fabric_layer:
            print(f"==>{layer.layer_id}:{layer.name}:{layer.kind}")
            res.append(fabric_layer)

    return res


def psd_to_fabric(psd):
    layers = parse_layers(psd._layers, 0, 0)
    viewbox = psd.viewbox
    fb = Fabric(layers, viewbox[0], viewbox[1], viewbox[2] - viewbox[0], viewbox[3] - viewbox[1])
    return fb


def common_parse(layer: Layer, fabric_layer: FabricLayer):
    fabric_layer.opacity = round(layer.opacity / 255, 2)
    # effect parse
    # 仅处理字体类型的effect，其他类型的effect 通过 composite 已经体现在图片上
    if len(layer.effects.items) > 0:
        for effect in layer.effects.items:
            effects_parse(effect, fabric_layer)
    # other parser append here


def effects_parse(effect, fabric_layer: FabricLayer):
    if isinstance(effect, Stroke) :
        stroke_parse(effect, fabric_layer)
    # if isinstance(effect, ColorOverlay):
    #     coloroverlay_parse(effect, fabric_layer)

